@@ -3,15 +3,16 @@ use warnings;
 
 package Net::FreshBooks::API::Base;
 BEGIN {
-  $Net::FreshBooks::API::Base::VERSION = '0.17';
+  $Net::FreshBooks::API::Base::VERSION = '0.18';
 }
 
 use Moose;
+#use namespace::autoclean;
+
 use Carp qw( carp croak );
 use Clone qw(clone);
 use Data::Dump qw( dump );
-
-#use Devel::SimpleTrace;
+use Net::FreshBooks::API::Error;
 use XML::LibXML ':libxml';
 use XML::Simple;
 use LWP::UserAgent;
@@ -26,17 +27,17 @@ my %plural_to_singular = (
     nesteds  => 'nested',    # for testing
 );
 
-has '_fb' => ( is => 'rw' );
-has '_sent_xml' => ( is => 'rw' );
+has 'error' => ( is => 'rw', isa => 'Net::FreshBooks::API::Error', lazy_build => 1 );
+
+has '_fb'                 => ( is => 'rw' );
+has '_sent_xml'           => ( is => 'rw' );
 
 sub new_from_node {
     my $class = shift;
     my $node  = shift;
 
     my $self = bless {}, $class;
-
     $self->_fill_in_from_node( $node );
-
     return $self;
 }
 
@@ -81,8 +82,7 @@ sub _fill_in_from_node {
                     map { $made_of->new_from_node( $_ ) }    #
                     grep { $_->nodeType eq XML_ELEMENT_NODE }
                     $match->childNodes();
-
-                $self->{$key} = \@new_objects;
+                    $self->{$key} = \@new_objects;
             }
             else {
                 $self->{$key}                                #
@@ -105,7 +105,7 @@ sub _fill_in_from_node {
 sub send_request {
     my $self = shift;
     my $args = shift;
-
+    
     my $fb     = $self->_fb;
     my $method = $args->{_method};
 
@@ -131,8 +131,6 @@ sub send_request {
 
     $fb->_log( debug => "Received response for $method" );
 
-    #carp "sending request\n";
-
     return $response_node;
 }
 
@@ -179,7 +177,8 @@ sub field_names_rw {
 
 sub parameters_to_request_xml {
     my $self       = shift;
-    my $parameters = clone( shift );
+    my $params = shift;
+    my $parameters = clone( $params );
 
     my $dom = XML::LibXML::Document->new( '1.0', 'utf-8' );
 
@@ -219,7 +218,7 @@ sub construct_element {
         elsif ( ref $val eq 'ARRAY' ) {
 
             my $singular_key = $plural_to_singular{$key}
-                || croak "couldnot convert '$key' to singular";
+                || croak "could not convert '$key' to singular";
 
             my $wrapper = XML::LibXML::Element->new( $key );
             $element->addChild( $wrapper );
@@ -258,7 +257,11 @@ sub response_xml_to_node {
 
     if ( $response_status ne 'ok' ) {
         my $msg = XMLin( $xml );
-        croak "FreshBooks server returned error: '$msg->{'error'}'";
+        my $error = "FreshBooks server returned error: '$msg->{error}'";
+        $self->error->handle_server_error( $error );
+    }
+    else {
+        $self->error->last_server_error( undef );
     }
 
     return $response;
@@ -297,11 +300,21 @@ sub send_xml_to_freshbooks {
 
     if ( !$response->is_success ) {
         croak "FreshBooks request failed: " . $response->status_line;
+        $self->error->handle_server_error( "FreshBooks request failed: " . $response->status_line );
     }
 
     return $response->content;
 }
 
+sub _build_error {
+    
+    my $self = shift;
+    my $error = Net::FreshBooks::API::Error->new;
+    return $error;
+    
+    
+}
+
 # When FreshBooks returns info on recurring items, it does not return the same
 # frequency values as the values it requests.  This method provides a lookup
 # table to fix this issue.
@@ -337,7 +350,7 @@ Net::FreshBooks::API::Base
 
 =head1 VERSION
 
-version 0.17
+version 0.18
 
 =head2 new_from_node
 
